#//-----------------------------------------------
#//TUSB-K02ADVZ用ドライバアクセス用API
#//Python 用モジュール
#//
#//2024_04_23　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

from ctypes import *

#definisions
c_int32_max = 2 ** 31 - 1

#Device DLL Loader ( cdecl )
tusbk02addll = cdll.LoadLibrary("TUSK02AD.DLL")

#######  dll function definitions (do not call directry from user codes)
tusbk02addll.TUSBK02AD_Device_Open.restype = c_short
tusbk02addll.TUSBK02AD_Device_Open.argtypes = (c_short,)
tusbk02addll.TUSBK02AD_Device_Close.restype = None
tusbk02addll.TUSBK02AD_Device_Close.argtypes = (c_short,)
tusbk02addll.TUSBK02AD_Ad_Single.restype = c_short
tusbk02addll.TUSBK02AD_Ad_Single.argtypes = (c_short,c_ubyte,POINTER(c_short))
tusbk02addll.TUSBK02AD_Ad_Start.restype = c_short
tusbk02addll.TUSBK02AD_Ad_Start.argtypes = (c_short,c_ubyte,c_ubyte,c_uint)
tusbk02addll.TUSBK02AD_Ad_Stop.restype = c_short
tusbk02addll.TUSBK02AD_Ad_Stop.argtypes = (c_short,)
tusbk02addll.TUSBK02AD_DIO_Dir.restype = c_short
tusbk02addll.TUSBK02AD_DIO_Dir.argtypes = (c_short,c_ubyte,c_ubyte)
tusbk02addll.TUSBK02AD_DIO_In.restype = c_short
tusbk02addll.TUSBK02AD_DIO_In.argtypes = (c_short,POINTER(c_ubyte))
tusbk02addll.TUSBK02AD_DIO_Out.restype = c_short
tusbk02addll.TUSBK02AD_DIO_Out.argtypes = (c_short,c_ubyte,c_ubyte)
tusbk02addll.TUSBK02AD_DIO_Status.restype = c_short
tusbk02addll.TUSBK02AD_DIO_Status.argtypes = (c_short,POINTER(c_ubyte))
tusbk02addll.TUSBK02AD_ExtTimer.restype = c_double
tusbk02addll.TUSBK02AD_ExtTimer.argtypes = (c_int,)
tusbk02addll.TUSBK02AD_ExtTimer_Set.restype = c_short
tusbk02addll.TUSBK02AD_ExtTimer_Set.argtypes = (c_short,c_int)
tusbk02addll.TUSBK02AD_Get_Data.restype = c_short
tusbk02addll.TUSBK02AD_Get_Data.argtypes = (c_short,POINTER(c_short),POINTER(c_short),POINTER(c_int))
tusbk02addll.TUSBK02AD_Get_Status.restype = c_short
tusbk02addll.TUSBK02AD_Get_Status.argtypes = (c_short,POINTER(c_ubyte),POINTER(c_int))
tusbk02addll.TUSBK02AD_IntClk.restype = c_double
tusbk02addll.TUSBK02AD_IntClk.argtypes = (c_int,)
tusbk02addll.TUSBK02AD_Trigger_Set.restype = c_short
tusbk02addll.TUSBK02AD_Trigger_Set.argtypes = (c_short,c_ubyte,c_short,c_short,c_int)
tusbk02addll.TUSBK02AD_SoftTrigger.restype = c_short
tusbk02addll.TUSBK02AD_SoftTrigger.argtypes = (c_short,)

#######  support functions (do not call directry from user codes)
def isint(n):
    return type(n) is int

def IsInRange(Val,Min,Max):
    if(isint(Val) == False ):
        return False
    elif(Val < Min):
        return False
    elif(Val > Max):
        return False
    else:
        return True

def IsValidId(DevId):
    return IsInRange(DevId,0,15)

# def functions

def TUSBK02AD_Device_Open(DevId):
    """Open Device
    
    Arguments:
    DevId           ----    Device Id 0~15
    
    Returns:
    int,            ----    Result code
    """
    if( IsValidId(DevId) ):
        return tusbk02addll.TUSBK02AD_Device_Open(DevId)
    else:
        return 8    #Invalid parameter

def TUSBK02AD_Device_Close(DevId):
    """Close Device

    Arguments:
    DevId           ----    Device Id 0~15
    """
    if( IsValidId(DevId) ):
        tusbk02addll.TUSBK02AD_Device_Close(DevId)
    else:
        return 8    #Invalid parameter

def TUSBK02AD_Ad_Single(DevId,Ch):
    """Get one AD Conversion data
    
    Arguments:
    DevId           ----    Device Id 0~15
    Ch              ----    Target Channel

    Returns:
    int             ----    Result code
    int             ----    conversion data
    """
    if( not IsValidId(DevId)): return 8,None
    if( not IsInRange(Ch,0,1)): return 8,None
    Data = c_short()
    ret = tusbk02addll.TUSBK02AD_Ad_Single(DevId,Ch,byref(Data))
    if ret == 0:
        return 0,Data.value
    else:
        return ret,None

def TUSBK02AD_Ad_Start(DevId,Ch,TrgCh,Clk):
    """Start Continuous Sampling
    
    Arguments:
    DevId           ----    Device Id 0~15
    Ch              ----    Select Target Channels
    TrgCh           ----    Select Trigger Channel 0 or 1
    Clk             ----    Set Sampling Interval
    
    Returns:
    int             ----    Result code
    """
    if( not IsValidId(DevId)): return 8
    if( not IsInRange(Ch,0,2)): return 8
    if( not IsInRange(TrgCh,0,1)): return 8
    if( not ( IsInRange(Clk,50000,1000000000) or IsInRange(Clk,0,0) ) ): return 8
    return tusbk02addll.TUSBK02AD_Ad_Start(DevId,Ch,TrgCh,Clk)

def TUSBK02AD_Ad_Stop(DevId):
    """Stop Continuous Sampling
    
    Arguments:
    DevId           ----    Device Id 0~15

    Returns:
    int             ----    Result code
    """
    if( IsValidId(DevId)):
        return tusbk02addll.TUSBK02AD_Ad_Stop(DevId)
    else:
        return 8
    
def TUSBK02AD_DIO_Dir(DevId,Bit,Dir):
    """Set Digital I/O bit direction
    
    Arguments:
    DevId           ----    Device Id 0~15
    Bit             ----    Target bit 0 or 1
    Dir             ----    Direction 0:in 1:out

    Returns:
    int             ----    Result code
    """
    if( not IsValidId(DevId)): return 8
    if( not IsInRange(Bit,0,1)): return 8
    if( not IsInRange(Dir,0,1)): return 8
    return tusbk02addll.TUSBK02AD_DIO_Dir(DevId,Bit,Dir)

def TUSBK02AD_DIO_In(DevId):
    """Read Digital Input Staus
    
    Arguments:
    DevId           ----    Device Id 0~15

    Returns:
    int             ----    Result code
    int             ----    Read input value
    """
    if( IsValidId(DevId)):
        Data = c_ubyte()
        ret = tusbk02addll.TUSBK02AD_DIO_In(DevId,byref(Data))
        if( ret== 0):
            return 0,Data.value
        else:
            return ret,None
    else:
        return 8,None

def TUSBK02AD_DIO_Out(DevId,Bit,Data):
    """Set Digital Output Level
    
    Arguments:
    DevId           ----    Device Id 0~15
    Bit             ----    Target bit 0 or 1
    Data            ----    Set Level 0:Low 1:High
    
    Returns:
    int             ----    Result code
    """
    if( not IsValidId(DevId)): return 8
    if( not IsInRange(Bit,0,1)): return 8
    if( not IsInRange(Data,0,1)): return 8
    return tusbk02addll.TUSBK02AD_DIO_Out(DevId,Bit,Data)

def TUSBK02AD_DIO_Status(DevId):
    """Read Digital I/O Port Status
    
    Arguments:
    DevId           ----    Device Id 0~15

    Returns:
    int             ----    Result code
    int             ----    Bit0 Status
    int             ----    Bit1 Status
    """
    if( IsValidId(DevId)):
        Data = (c_ubyte * 2)()
        p_Data = cast(pointer(Data),POINTER(c_ubyte))
        ret = tusbk02addll.TUSBK02AD_DIO_Status(DevId,p_Data)
        if( ret== 0):
            return 0,Data[0],Data[1]
        else:
            return ret,None,None
    else:
        return 8,None,None
    
def TUSBK02AD_ExtTimer(Clk):
    """Calculate ext timer frequency from interval setting number
    
    Arguments:
    Clk             ----    Clock Interval Number 1000~5000000000 [ns]

    Returns:
    double          ----    calculated frequency or 0.0(invalid Clk range)
    """
    if( IsInRange(Clk,1000,5000000000)):
        return tusbk02addll.TUSBK02AD_ExtTimer(Clk)
    else:
        return 0.0

def TUSBK02AD_ExtTimer_Set(DevId,Clk):
    """Set Timer Clock Interval
    
    Arguments:
    DevId           ----    Device Id 0~15
    Clk             ----    Clock Interval Number 1000~5000000000 [ns]

    Returns:
    int             ----    Result code
    """
    if( not IsValidId(DevId)): return 8
    if( not IsInRange(Clk,1000,5000000000)): return 8
    return tusbk02addll.TUSBK02AD_ExtTimer_Set(DevId,Clk)

def TUSBK02AD_Get_Data(DevId,ReqLen):
    """Get Continuous Sampling Datas
    
    Arguments:
    DevId           ----    Device Id 0~15
    ReqLen          ----    Request Data Length

    Returns:
    int             ----    Result code
    tuple(int)      ----    Ch1 Data tuple
    tuple(int)      ----    Ch2 Data tuple
    int             ----    Number of data acquired
    """
    if( not IsValidId(DevId)): return 8,None,None,None
    if( not IsInRange(ReqLen,0,c_int32_max)): return 8,None,None,None
    Data1 = ( c_short * ReqLen)()
    Data2 = ( c_short * ReqLen)()
    Leng = c_int(ReqLen)
    p_Data1 = cast(pointer(Data1),POINTER(c_short))
    p_Data2 = cast(pointer(Data2),POINTER(c_short))
    ret = tusbk02addll.TUSBK02AD_Get_Data(DevId,p_Data1,p_Data2,byref(Leng))
    if( ret == 0):
        ResultL = Leng.value
        RData1 = tuple(int(x) for x in Data1[:ResultL])
        RData2 = tuple(int(x) for x in Data2[:ResultL])
        return 0,RData1,RData2,ResultL
    else:
        return 8,None,None,None

def TUSBK02AD_Get_Status(DevId):
    """Get Sampling Status
    
    Arguments:
    DevId           ----    Device Id 0~15

    Returns:
    int             ----    Result code
    int             ----    Status code
    int             ----    Data Memory Length
    """
    if( IsValidId(DevId)):
        Status = c_ubyte()
        Length = c_int()
        ret = tusbk02addll.TUSBK02AD_Get_Status(DevId,byref(Status),byref(Length))
        if ret == 0:
            return 0,Status.value,Length.value
        else:
            return ret,None,None,None
    return 8,None,None,None

def TUSBK02AD_IntClk(Clk):
    """Calculate Sampling Frequency from clk interval setting number
    
    Arguments:
    Clk             ----    Clock interval setting 50,000 ~ 1,000,000,000 [ns]

    Returns:
    double          ----    calculated frequency or 0.0(invalid Clk range)
    """
    if( IsInRange(Clk,50000,1000000000)):
        return tusbk02addll.TUSBK02AD_IntClk(Clk)
    else:
        return 0.0

def TUSBK02AD_Trigger_Set(DevId,TrgType,Level,Hys,PreLen):
    """Set Sampling Conditions
    
    Arguments:
    DevId           ----    Device Id 0~15
    TrgType         ----    Trigger type code
    Level           ----    Analog Trigger Level
    Hys             ----    Analog Trigger Hysteresis
    PreLen          ----    Pre Trigger Length

    Returns:
    int             ----    Result code
    """
    if( not IsValidId(DevId)): return 8
    if( not IsInRange(TrgType,0,9)): return 8
    if( not IsInRange(Level,0,4095)): return 8
    if( not IsInRange(Hys,0,41)): return 8
    if( not IsInRange(PreLen,0,1048576)): return 8
    return tusbk02addll.TUSBK02AD_Trigger_Set(DevId,TrgType,Level,Hys,PreLen)

def TUSBK02AD_SoftTrigger(DevId):
    """Generate Software Trigger
    
    Arguments:
    DevId           ----    Device Id 0~15

    Returns:
    int             ----    Result code
    """
    if IsValidId(DevId):
        return tusbk02addll.TUSBK02AD_SoftTrigger(DevId)
    else:
        return 8

# Error Messages from return coddes
ErrStrDic = {
            0:"正常終了しました",
            1:"ID番号が不正です",
            2:"ドライバが初期化されていません",
            3:"すでにデバイスはオープンされています",
            4:"接続されている台数が多すぎます",
            5:"オープンできませんでした",
            6:"デバイスがみつかりません",
            8:"パラメータエラーです",
            9:"USB通信エラーです",
            11:"連続取り込み動作中です",
            13:"開始されていません",
            14:"メモリオーバーフローです",
            15:"データ並びエラーです",
            99:"その他のエラーです"
}
def GetErrMessage(retcode):
    if( retcode in ErrStrDic):
        return ErrStrDic[retcode]
    else:
        return "不明なエラーです"
    