#//-----------------------------------------------
#//TUSB-K02ADVZ用サンプルプログラム
#//Python 用モジュール
#//
#//2024_04_23　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------
import TUSBK02AD
import sys

from SpFuncs import *
from functools import partial
from tkinter import filedialog

class Tusbk02ad_Device:
    """TUSB-K02ADVZ デバイス操作クラス フォーム"""

    DevId = None    # デバイスID
    mon_t = None    # DIOモニタータイマー
    smpl_t = None   # AD連続サンプリングタイマー
    TotalLen = None # 連続サンプリング時サンプリング総個数
    DFile = None    # サンプリングデータ保存ファイル

    def __init__(self,DevId):
        """イニシャライザ
        
        arguments:
        DevId           ----    Device id 0~15
        """
        ret = TUSBK02AD.TUSBK02AD_Device_Open(DevId)
        if ret == 0:    # 正常にオープン出来た場合
            self.DevId = DevId                                      # デバイスIDをクラスメンバ変数に保持する
            self.form.protocol("WM_DELETE_WINDOW", self.DevClose)   # フォームが閉じられる時の処理を指定する
            if self.DevInit():                                      # デバイスの初期化処理
                self.mon_t = intTT(0.5,self.Monitor)                    # モニタータイマー作成、コールバック関数登録
                self.mon_t.Start()                                      # モニタータイマースレッド開始
                self.WidgetState("Stop")
            else:
                TUSBK02AD.TUSBK02AD_Device_Close(DevId)                 # デバイス初期化に失敗したらデバイスを閉じる
                self.form.destroy()                                     # フォームも閉じる
        else:
            ShowInfo(TUSBK02AD.GetErrMessage(ret))                  # 失敗エラーメッセージ表示
            self.form.destroy()                                     # フォームを閉じる

    def DevInit(self):
        """デバイスの状態を確認して画面表示に反映する"""
        ret = TUSBK02AD.TUSBK02AD_DIO_Status(self.DevId)    # DIO 入出力状態の確認
        if ret[0] != 0:
            ShowInfo(TUSBK02AD.GetErrMessage(ret[0]))
            return False
        self.cmbb_dio[0].current(ret[1])    # DIO現在状態を表示に反映する
        self.cmbb_dio[1].current(ret[2])
        # 以下ボタンやコンボボックス等のイベントハンドラを登録する処理
        self.cmbb_dio[0].bind("<<ComboboxSelected>>", lambda e: self.Dio_Change_on(0))
        self.cmbb_dio[1].bind("<<ComboboxSelected>>", lambda e: self.Dio_Change_on(1))
        self.txt_extclk.bind("<KeyRelease>",lambda e: self.ExClk_Change_on())
        self.btn_exclk.config(command=lambda: self.Clock_out_on())
        self.btn_ad.config(command=lambda: self.Convert_on())
        self.cmbb_trgtype.bind("<<ComboboxSelected>>",lambda e: self.Trg_Type_on())
        self.rbtn[0].config(command=lambda: self.Clk_ExIn_on())
        self.rbtn[1].config(command=lambda: self.Clk_ExIn_on())
        self.txt_reqclk.bind("<KeyRelease>",lambda e: self.Request_Clock_Change_on())
        self.btn_start.config(command=lambda: self.Start_on())
        self.btn_stop.config(command=lambda: self.Stop_on())
        self.btn_trigger.config(command=lambda: self.Soft_Trigger_on())
        return True
    
    def DevClose(self):
        """デバイスを閉じる時の処理"""
        if self.DevId != None:
            self.mon_t.Stop()                               # モニタタイマー終了
            TUSBK02AD.TUSBK02AD_Device_Close(self.DevId)    # デバイスクローズ
            self.form.destroy()                             # フォームを閉じる
            self.DevId = None
    
    def WidgetState(self,stat):
        """状態に応じてウィジットの表示を切り替える処理
        
        Arguments
        stat            ----    設定状態を表す文字列

        'Stop' - デバイスが接続されていて連続取り込み実行していない時
        'Run' - デバイスが接続されていて連続取り込みを実行している時
        """
        EnStop = tk.NORMAL if( stat == 'Stop' ) else tk.DISABLED
        EnRun = tk.NORMAL if( stat == 'Run' ) else tk.DISABLED
        self.txt_extclk['state'] = EnStop
        self.btn_exclk['state'] = EnStop
        self.btn_ad['state'] = EnStop
        self.cmbb_trgtype['state'] = EnStop
        self.txt_prelen['state'] = EnStop
        self.TrgWidgetState(stat)
        self.chkb[0]['state'] = EnStop
        self.chkb[1]['state'] = EnStop
        self.rbtn[0]['state'] = EnStop
        self.rbtn[1]['state'] = EnStop
        self.IntClockWidgetState()
        self.btn_start['state'] = EnStop
        self.btn_stop['state'] = EnRun
        self.btn_trigger['state'] = tk.DISABLED if ( self.cmbb_trgtype.get() != 'Soft' ) else EnRun
    
    def TrgWidgetState(self,stat):
        """トリガコントロール関連ウィジットの表示を切り替える処理
        
        arguements:
        stat            ----    "stop" pr "run"
        """
        type = self.cmbb_trgtype.current()
        if stat == "run":
            self.cmbb_trgch['state'] = tk.DISABLED
            self.txt_trglev['state'] = tk.DISABLED
            self.txt_hys['state'] = tk.DISABLED
        elif type <= 3:
            self.cmbb_trgch['state'] = tk.DISABLED
            self.txt_trglev['state'] = tk.DISABLED
            self.txt_hys['state'] = tk.DISABLED
        elif type <= 5:
            self.cmbb_trgch['state'] = tk.NORMAL
            self.txt_trglev['state'] = tk.NORMAL
            self.txt_hys['state'] = tk.NORMAL
        else:
            self.cmbb_trgch['state'] = tk.NORMAL
            self.txt_trglev['state'] = tk.NORMAL
            self.txt_hys['state'] = tk.DISABLED
    
    def IntClockWidgetState(self):
        """内部クロック設定ウィジットの表示を切り替える処理"""
        if self.RbtnV.get() == 0: # External selected
            self.txt_reqclk['state'] = tk.DISABLED
        else: # Internal selected
            self.txt_reqclk['state'] = tk.NORMAL

    # timer -------------------------------------
    def Monitor(self):
        """モニタータイマー DIO入力チェック"""
        ret = TUSBK02AD.TUSBK02AD_DIO_In(self.DevId)                        # ディジタル入力を読み込む
        if ret[0] == 0:
            self.lbl_dio[0]['text'] = "High" if bitChk(ret[1],0) else "Low"
            self.lbl_dio[1]['text'] = "High" if bitChk(ret[1],1) else "Low"

    def SampleT(self):
        """サンプリングタイマーコールバック"""
        self.DataReadAndSave()

    # 補助的関数/////////////////////////
    def Dio_Set(self,Bit,Dir,Dat):
        """DIOの設定関数
        
        Arguments:
        Bit             ----    設定ビット 0 or 1
        Dir             ----    入出力方向 0(IN) or 1(OUT)
        Dat             ----    出力データ 0(Low) or 1(High)
        """
        if Dir == 1:    # 出力であれば
            ret = TUSBK02AD.TUSBK02AD_DIO_Out(self.DevId,Bit,Dat)   # ディジタル出力値を設定
            if ret != 0:
                ShowInfo(TUSBK02AD.GetErrMessage(ret))
                return
        ret = TUSBK02AD.TUSBK02AD_DIO_Dir(self.DevId,Bit,Dir)       # 入出力方向を設定
        if ret != 0:
            ShowInfo(TUSBK02AD.GetErrMessage(ret))

    #/// ボタン等ウィジットの操作イベントハンドラ /////////////////////////
    def Dio_Change_on(self,Bit):
        """DIOの設定が変更された"""
        ststr = self.cmbb_dio[Bit].get()
        if ststr == 'INPUT':
            self.Dio_Set(Bit,0,0)
        elif ststr == 'LOW':
            self.Dio_Set(Bit,1,0)
        elif ststr == 'HIGH':
            self.Dio_Set(Bit,1,1)
        elif ststr == 'CLOCK':
            """ディジタルIOの設定ではクロックは選択しない。外部クロックで
            出力設定をすると自動的にクロック出力になる。手動でクロックが
            選択された時は、現在のDIOの状態を確認してそれを出力状態
            コンボボックスに設定しなおす"""
            if Bit == 1:    # クロックがあるのはDIO1のみ
                ret = TUSBK02AD.TUSBK02AD_DIO_Status(self.DevId)    # 現在の出力状態の確認
                if ret[0] != 0:
                    ShowInfo(TUSBK02AD.GetErrMessage(ret[0]))
                    return False
                if ret[2] != 3:                                     # クロック設定以外の時はコンボボックスの選択をそれに合わせる
                    self.cmbb_dio[Bit].current(ret[2])

    def ExClk_Change_on(self):
        """外部出力クロックの設定値が変更された"""
        Clk = ForceToInt(self.txt_extclk.get())
        ret = TUSBK02AD.TUSBK02AD_ExtTimer(Clk)                 # 外部クロックに設定されうる値を取得する
        if ret == 0.0:
            self.lbl_exclk['text'] = "Invalid Divide Setting"
        else:
            self.lbl_exclk['text'] = str(ret)

    def Trg_Type_on(self):
        """トリガタイプが変更された"""
        self.TrgWidgetState(0)

    def Clk_ExIn_on(self):
        """内部クロック、外部クロックの選択が変更された"""
        self.IntClockWidgetState()

    def Clock_out_on(self):
        """クロック出力設定"""
        Clk = ForceToInt(self.txt_extclk.get())
        ret = TUSBK02AD.TUSBK02AD_ExtTimer_Set(self.DevId,Clk)  # 外部クロック出力設定
        if ret != 0:
            ShowInfo(TUSBK02AD.GetErrMessage(ret))
            return False
        else:
            self.cmbb_dio[1].current(3)

    def Convert_on(self):
        """AD単一変換実行"""
        for ch in range(2):
            ret = TUSBK02AD.TUSBK02AD_Ad_Single(self.DevId,ch)  # 指定チャンネルの入力値を1つ取得する(連続取り込みデータではない)
            if ret[0] != 0:
                ShowInfo(TUSBK02AD.GetErrMessage(ret[0]))
                return
            dataD = (ret[1]-2048.0) / 2048.0 * 2.5              # 変換値から電圧値への変換式
            self.lbl_ad[ch]['text'] = format(dataD,".3f")

    def Request_Clock_Change_on(self):
        """内部クロック設定値が変更された"""
        Clk = ForceToInt(self.txt_reqclk.get())
        ret = TUSBK02AD.TUSBK02AD_IntClk(Clk)                   # 内部クロックに設定されうる値を取得する
        if ret == 0.0:
            self.lbl_inclk['text'] = "Invalid Clock Setting"
        else:
            self.lbl_inclk['text'] = format(ret,".0f")

    def Start_on(self):
        """AD連続変換開始"""
        # 連続変換に必要な設定値をフォームウィジェットから取得する
        TrgType = self.cmbb_trgtype.current()       # トリガの種類
        TrgCh = self.cmbb_trgch.current()           # トリガチャンネル
        tmpD = ForceToFloat(self.txt_trglev.get())  # トリガレベル電圧
        Level = round((tmpD + 2.5) / 5.0 * 4096.0)  # トリガレベル設定値への変換
        tmpD = ForceToFloat(self.txt_hys.get())     # ヒステリシス電圧
        Hys = round(tmpD / 5000.0 * 4096.0)         # ヒステリシス設定値への変換
        PreLen = ForceToInt(self.txt_prelen.get())  # プレトリガ長
        # 取り込み対象チャンネルの確認
        if self.ChkbV[0].get():     # ch1が取得対象  
            if self.ChkbV[1].get():     # ch2が取得対象
                Ch = 2  # Ch1 & Ch2
            else:                       # ch2は取得対象ではない
                Ch = 0  # Ch1
        else:                       # ch1は取得対象ではない    
            if self.ChkbV[1].get():     # ch2が取得対象
                Ch = 1  # Ch2
            else:                       # ch2は取得対象ではない
                return  # No Ch selected　意味が無いので終了リターン
    
        Clk = ForceToInt(self.txt_reqclk.get())     # クロック設定

        if self.SaveFileOpen() == False: return     # 保存先ファイルを指定作成
        ret = TUSBK02AD.TUSBK02AD_Trigger_Set(self.DevId,TrgType,Level,Hys,PreLen)  # トリガ設定
        if ret != 0:
            ShowInfo(TUSBK02AD.GetErrMessage(ret))
            return False
        ret = TUSBK02AD.TUSBK02AD_Ad_Start(self.DevId,Ch,TrgCh,Clk)                 # AD変換開始
        if ret != 0:
            ShowInfo(TUSBK02AD.GetErrMessage(ret))
            return False
        self.TotalLen = 0                           # データ取り込み済み長を0個に初期化
        self.smpl_t = intTT(0.3,self.SampleT)       # サンプリングタイマー作成、コールバック関数指定
        self.smpl_t.Start()                         # サンプリングタイマースタート
        self.WidgetState("Run")                     # フォームウィジェット表示をRun状態にする
    
    def Stop_on(self):
        """AD連続変換停止"""
        ret = TUSBK02AD.TUSBK02AD_Ad_Stop(self.DevId)   # サンプリング停止
        if ret != 0:
            ShowInfo(TUSBK02AD.GetErrMessage(ret))
        self.smpl_t.Stop()                              # サンプリングタイマー停止
        self.DataReadAndSave()                          # 取り込み済みデータをファイルへ書き込む
        self.SaveFileClose()                            # ファイルを閉じる
        self.WidgetState("Stop")                        # フォームウィジェット表示をStop状態にする
    
    def Soft_Trigger_on(self):
        """トリガ発生"""
        ret = TUSBK02AD.TUSBK02AD_SoftTrigger(self.DevId)   # ソフトウェアトリガ発生
        if ret != 0:
            ShowInfo(TUSBK02AD.GetErrMessage(ret))

    # ファイル入出力関連処理 ----------------------------------
    def SaveFileOpen(self):
        """保存ファイルの選択とオープン
        
        Returns:
        bool            ----    成功 True 失敗 False
        """
        filename = filedialog.asksaveasfilename(
            title = "保存ファイルを指定してください",
            filetypes = [("データファイル","*.csv")],
            defaultextension = "csv"
            )
        if len(filename) == 0 : return False
        try:
            self.DFile = open(filename,"w")     # ファイルを開く
        except OSError as e:
            ShowInfo(e)
            return False
        return True

    def SaveFileClose(self):
        """保存ファイルを閉じる"""
        if self.DFile != None:
            self.DFile.close()                  # ファイルを閉じる
            self.DFile = None

    def SaveFileWrite(self,Data):
        """保存ファイルにデータを保存する
        
        Arguments:
        Data            ----    データタプル(or リスト)のタプル(or リスト)
                                2ch * データ数分のリスト(or タプル) ((),())
                                ch1とch2は同じ長さとする事
        """
        leng = len(Data[0])                 # データの個数を確認
        if leng != len(Data[1]):return      # Data[0]とData[1]は同じ長さ(要素数)でなければならない
        if leng == 0:return                 # 0個の場合も直ぐにリターン
        if self.DFile == None: return       # データファイルが無しならリターン
        for n in range(leng):               # 1行ずつ保存する
            self.DFile.write(str(Data[0][n]) + "," + str(Data[1][n])+"\n")

    
    def DataReadAndSave(self):
        """デバイスからデータを読み出してファイルに保存する"""
        ret = TUSBK02AD.TUSBK02AD_Get_Status(self.DevId)    # 連続取り込み状態確認
        if ret[0] != 0:
            # ShowInfo(TUSBK02AD.GetErrMessage(ret))
            return
        Status = ret[1]     # 取り込み状態
        Leng = ret[2]       # 取り込み保持データ数
        if Leng > 0:        # データが1個以上あれば
            ret = TUSBK02AD.TUSBK02AD_Get_Data(self.DevId,Leng) # データを取得する
            if ret[0] != 0:
                return
            self.SaveFileWrite((ret[1],ret[2]))                 # データを保存
            self.TotalLen += Leng                               # 保存データ数を加算記録
            self.lbl_smplst['text'] = "Status = " + str(Status) + "   TotalLeng = " + str(self.TotalLen)

    # フォームとウィジェットを生成する処理---------------------------
    form = tk.Tk()
    form.geometry('440x484')
    form.title('TUSB-K02ADVZ Sample Program')

    frame = GenList(4,None)     # フレーム(枠)

    # DIOフレーム
    DioSettDictCh1= {
        "INPUT":0,
        "LOW":1,
        "HIGH":2,
    }
    DioSettDictCh2= {
        "INPUT":0,
        "LOW":1,
        "HIGH":2,
        "CLOCK":3,
    }
    lbl_dio = GenList(2,None)   # DIO状態表示ラベル
    cmbb_dio = GenList(2,None)  # DIO設定コンボボックス
    frame[0] = GetLFrame(form,'DIO',0.95,0.12,0.025,0.02)
    GetLabel(frame[0],"D0",0.05,0.5)
    GetLabel(frame[0],"D1",0.55,0.5)
    lbl_dio[0] = GetLabelDisp(frame[0],"",0.15,0.15,0.5)
    lbl_dio[1] = GetLabelDisp(frame[0],"",0.15,0.65,0.5)
    cmbb_dio[0] = GetComboBox(frame[0],list(DioSettDictCh1.keys()),0.15,0.33,0.5)
    cmbb_dio[1] = GetComboBox(frame[0],list(DioSettDictCh2.keys()),0.15,0.83,0.5)

    # Clock Output フレーム
    txt_exclk = None            # クロック出力設定入力
    lbl_exclk = None            # クロック出力設定値表示ラベル
    btn_exclk = None            # クロック出力ボタン
    frame[1] = GetLFrame(form,'Clock Output',0.95,0.14,0.025,0.14)
    GetLabel(frame[1],"Request",0.25,0.2)
    GetLabel(frame[1],"ns",0.75,0.2)
    GetLabel(frame[1],"Actual",0.25,0.7)
    GetLabel(frame[1],"ns",0.75,0.7)
    txt_extclk = GetEntry(frame[1],"",0.33,0.4,0.2)
    lbl_exclk = GetLabelDisp(frame[1],"",0.33,0.4,0.7)
    btn_exclk = GetButton(frame[1],"Clock Out",None,0.2,0.12,0.45)

    # AD Sample フレーム
    lbl_ad = GenList(2,None)    # AD変換値表示ラベル
    btn_ad = None               # シングルAD変換ボタン
    frame[2] = GetLFrame(form,'AD Sample',0.95,0.14,0.025,0.28)
    GetLabel(frame[2],"CH1",0.3,0.15)
    GetLabel(frame[2],"CH2",0.65,0.15)
    GetLabel(frame[2],"[V]",0.45,0.65)
    GetLabel(frame[2],"[V]",0.8,0.65)
    lbl_ad[0] = GetLabelDisp(frame[2],"",0.15,0.3,0.65)
    lbl_ad[1] = GetLabelDisp(frame[2],"",0.15,0.65,0.65)
    btn_ad = GetButton(frame[2],"Convert",None,0.2,0.12,0.5)

    # Recording フレーム
    TriggerTypeDict= {
        "Soft":0,
        "Ext UpEdge":1,
        "Ext DownEdge":2,
        "Ext Edge":3,
        "Signal Up":4,
        "Signal Down":5,
        "Signal Upper":6,
        "Signal Lower":7,
        "Signal Inside":8,
        "Signal Outside":9,
    }
    TriggerChDict= {
        "Ch1":0,
        "Ch2":1,
    }
    ClockSelDict= {
        "External":0,
        "Internal":1,
    }
    chkb = GenList(2,None)      # チェックボックス
    rbtn = GenList(2,None)      # ラジオボタン
    ChkbV = GenTkVarList(2,tk.BooleanVar,False)  # チェックボックスのチェック状態
    RbtnV = tk.IntVar(value=0)                         # ラジオボタンの選択
    lbl_inclk = None            # サンプリングクロック設定値表示ラベル
    lbl_smplst = None           # サンプリング状態表示ラベル
    cmbb_trgtype = None         # トリガタイプ選択コンボボックス
    cmbb_trgch = None           # トリガチャンネル選択コンボボックス
    txt_trglev = None           # トリガレベル設定入力
    txt_hys = None              # ヒステリシスレベル入力
    txt_prelen = None           # プレトリガ長入力
    txt_reqclk = None           # サンプリングクロック設定入力
    btn_start = None            # サンプリング開始ボタン
    btn_stop = None             # サンプリング停止ボタン
    btn_trigger = None          # ソフトウェアトリガボタン
    frame[3] = GetLFrame(form,'AD Sample',0.95,0.55,0.025,0.42)
    ry = 0.062                  # 高さ位置調整用変数
    ypitch = 0.12               # このフレーム各行の高さピッチ
    GetLabel(frame[3],"Trigger Type",0.05,ry)
    GetLabel(frame[3],"Trigger Ch",0.55,ry)
    cmbb_trgtype = GetComboBox(frame[3],list(TriggerTypeDict.keys()),0.28,0.25,ry)
    cmbb_trgch = GetComboBox(frame[3],list(TriggerChDict.keys()),0.15,0.7,ry)
    ry += ypitch
    GetLabel(frame[3],"Trigger Level",0.05,ry)
    GetLabel(frame[3],"[V]",0.39,ry)
    GetLabel(frame[3],"Hysteresis",0.55,ry)
    GetLabel(frame[3],"[mV]",0.85,ry)
    txt_trglev = GetEntry(frame[3],"0",0.12,0.25,ry)
    txt_hys = GetEntry(frame[3],"0",0.12,0.7,ry)
    ry += ypitch
    GetLabel(frame[3],"PreTriggerLen",0.05,ry)
    GetLabel(frame[3],"max(1,048,576)",0.65,ry)
    txt_prelen = GetEntry(frame[3],"0",0.38,0.25,ry)
    ry += ypitch
    GetLabel(frame[3],"Ch. Select",0.05,ry)
    chkb[0] = GetChkBox(frame[3],"Ch1",0.25,ry,ChkbV[0])
    chkb[1] = GetChkBox(frame[3],"Ch2",0.35,ry,ChkbV[1])
    ry += ypitch
    GetLabel(frame[3],"Clock",0.05,ry)
    GetLabel(frame[3],"Request",0.5,ry)
    GetLabel(frame[3],"ns",0.9,ry)
    rbtn = GetRadioButtons(frame[3],ClockSelDict,0.15,ry,0.15,0.0,RbtnV)
    txt_reqclk = GetEntry(frame[3],"",0.25,0.63,ry)
    ry += ypitch
    GetLabel(frame[3],"Actual",0.5,ry)
    GetLabel(frame[3],"ns",0.9,ry)
    lbl_inclk = GetLabelDisp(frame[3],"",0.25,0.63,ry)
    ry += ypitch
    lbl_smplst = GetLabelDisp(frame[3],"",0.9,0.05,ry)
    ry += ypitch
    btn_start = GetButton(frame[3],"Start",None,0.2,0.12,ry)
    btn_stop = GetButton(frame[3],"Stop",None,0.2,0.5,ry)
    btn_trigger = GetButton(frame[3],"Soft Trigger",None,0.2,0.88,ry)

# 実行部 ---------------------------------
if __name__=="__main__":
    DevId = 0
    if( len(sys.argv) >= 2 ):
        if sys.argv[1].isdigit():
            DevId = int(sys.argv[1])    # コマンドライン引数でID設定
    tmp = Tusbk02ad_Device(DevId=DevId)
    tmp.form.mainloop()

